//*****************************************
// Microdot V1
// Copyright 2006 Philip Pulle
//                RGB Sunset Productions
// For private use only, not for commercial
// use. 
// ****************************************

#include <system.h>
#include "display.h"
#include "control.h"
#include "rand.h"
#include "t1rtc.h"

extern unsigned char nRTCSecs;
extern unsigned char nRTCMins;
extern unsigned char nRTCHours;
extern unsigned char nRTCTick;

//                              LED1  LED2  LED3  LED4  LED5  LED6  LED7  LED8  LED9  LED10 LED11 LED12 LED13 LED14 LED15 LED16 LED17 LED18 LED19 LED20 LED21 LED22 LED23 LED24 LED25 LED26 LED27 LED28 LED29 LED30
unsigned char LEDS_TRISA[31] = { 0xfd, 0xee, 0xf7, 0xfd, 0xfc, 0x6f, 0xef, 0xfc, 0x6f, 0xef, 0x7d, 0x7f, 0xed, 0x7d, 0x7f, 0xed, 0x77, 0x7e, 0xf6, 0x77, 0x7e, 0xf6, 0xf5, 0xfe, 0xe7, 0xf5, 0xfe, 0xe7, 0xf7, 0xee, 0xff};
unsigned char LEDS_PORTA[31] = { 0x02, 0x01, 0x08, 0x00, 0x01, 0x80, 0x00, 0x02, 0x10, 0x10, 0x02, 0x00, 0x02, 0x80, 0x80, 0x10, 0x80, 0x80, 0x01, 0x08, 0x01, 0x08, 0x08, 0x00, 0x08, 0x02, 0x01, 0x10, 0x00, 0x10, 0x00};
unsigned char LEDS_TRISB[31] = { 0xfb, 0xff, 0xfb, 0xfb, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xfb, 0xff, 0xfb, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xfb, 0xff, 0xfb, 0xff, 0xff};
unsigned char LEDS_PORTB[31] = { 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00};
unsigned char nLeds[30];
unsigned char nCurrentLed=0;

//Variables showing the number of LEDs to light in each colour group
unsigned char nG4,nG3,nG2,nG1;


//FUNCTION to initialise the display
void init_display(void)
{
	unsigned char i;
	
	//Turn all LEDs off
	for (i=0;i<30;i++) nLeds[i] = LED_OFF;
	
	//Setup TMR0 to update the display
	
	option_reg = 0b10001000;	//RBPU disbled, T0 internal
	set_bit(intcon, TMR0IE);	//enable the display interrupt
	set_bit(intcon, GIE);		//enable all interrupts
}

//FUNCTION to display a wipe across the LED array
void display_wipe(void)
{
	unsigned char i;
	for(i = 0; i<30; i++) 
	{
		led_on(i);
		delay_ms(10);
	}
	for(i = 0; i<30; i++) 
	{
		led_off(i);
		delay_ms(10);
	}
}

//FUNCTION to display randomly selected LEDs in the array
void display_random(void)
{
	unsigned char i,j,k;
	delay_ms(400);
	for (i = 0; i<255; i++)
	{
		j = rand() % 30;
		k = rand();
		if (k > 128) led_on(j); else led_off(j);
		delay_ms(50);
		if (!test_bit(SW_PORT, SW3_PIN)) break;
	}
}

//FUNCTION to gradually increase the strength of a 'bar graph' in about 20 seconds, then blink when maxed
void display_bar(void)
{
	unsigned char i,j,k,nBar, nLed;
	delay_ms(400);
		
	for (i = 0; i< 80; i++)
	{
		for(j=0;j<30;j++) nLeds[j] = LED_OFF;
		for (nBar = 0; nBar<3; nBar++)
		{
			j = i / 9 + 2;
			k = rand() % j;
			if (k>9) k = 9;
			nLed = nBar;
			for(j = 0; j<k;j++)
			{
				led_on(nLed);
				nLed = nLed + 3;
			}
		}
		if (!test_bit(SW_PORT, SW3_PIN)) break;
		delay_ms(40);
	}
	while(test_bit(SW_PORT, SW3_PIN))
	{
		flash_display();
		delay_ms(300);
	}
	delay_ms(300);
}

//FUNCTION to set a countdown timer
void display_timer(void)
{
	unsigned short nSecs;
	
	nG4 = 0;		//default to 3 minute egg timer
	nG3 = 3;
	nG2 = 0;
	nG1 = 0;
	show_time(DISP_LINEAR);	//show countdown timer in linear display mode as it's easier to read
	set_time();				//allow user to set the countdown time (in minutes/seconds)
	flash_display();		//signal timer has been set
	show_time(DISP_LINEAR);
	nSecs = nG4 * 10 + nG3;			//calculate number of seconds from user setting
	nSecs = nSecs * 60;
	nSecs = nSecs + nG2 * 10 + nG1;
	RTCTick(nSecs);					//and reset the timer tick variable
	while (nRTCTick>0)
	{
		if (nSecs != nRTCTick)
		{
			nSecs = (nRTCTick / 60);	//upon each tick, show the updated tick variable counting down
			nG4 = nSecs / 10;
			nG3 = nSecs % 10;
			nSecs = nRTCTick % 60;
			nG2 = nSecs / 10;
			nG1 = nSecs % 10;
			show_time(DISP_LINEAR);
			nSecs = nRTCTick;
		}
		if (!test_bit(SW_PORT, SW3_PIN)) //abort if SW3 pressed
		{
			delay_ms(300);
			break;
		}
	}
	while(test_bit(SW_PORT, SW3_PIN))
	{
		flash_display();	//keep flashing display to show timer complete
		delay_ms(300);
	}
	calc_time();
	show_time(DISP_RANDOM);	//show time as per normal
}

//FUNCTION to turn on each led consequtively
void led_test(void)
{
	unsigned char i;
	for(i=0;i<30;i++)
	{
		trisa = 0xff;
		trisb = 0xff;
		porta = LEDS_PORTA[i];
		portb = LEDS_PORTB[i];
		trisa = LEDS_TRISA[i];
		trisb = LEDS_TRISB[i];
		delay_ms(50);
	}
			

	
}

//FUNCTION to turn on a LED on the display
//INPUTS: 
//	nLed :- number of LED to turn on
void led_on(unsigned char nLed)
{
	nLeds[nLed] = LED_ON;
}

//FUNCTION to turn on a LED on the display
//INPUTS: 
//	nLed :- number of LED to turn on
void led_off(unsigned char nLed)
{
	nLeds[nLed] = LED_OFF;
}

//FUNCTION to select the next LED in the list and either turn it on
//or leave it off according to the LED table. Should be called periodically
//by the interrupt function
void update_display(void)
{
	unsigned char i;
	
	//Move to next LED
	nCurrentLed++;
	if(nCurrentLed>29) nCurrentLed = 0;	//rollover if necessary
	
	porta = 0;		//turn off previous leds what ever they may be
	portb = 0;
	
	if (nLeds[nCurrentLed]==LED_ON)
		i = nCurrentLed;
	else
		i = ALL_OFF;
		
	trisa = LEDS_TRISA[i];
	trisb = LEDS_TRISB[i];
	porta = LEDS_PORTA[i];
	portb = LEDS_PORTB[i];	

}


//FUNCTION to calculate time from RTC
void calc_time(void)
{
	unsigned char i,j;

	i = nRTCHours; j = nRTCMins;
	nG4 = i / 10 ;
	nG3 = i % 10 ;
	
	nG2 = j / 10 ;
	nG1 = j % 10 ;
}

//FUNCTION to show the time
//INPUTS:
//	nG4..nG1 	:- number of LEDs in colour groups (red, amber, green, blue)
//	nMode 		:- DISP_RANDOM, DISP_LINEAR show either random distribution (for time) of 
//				Leds or a linear distribution (for counting, stopwatch)
void show_time(unsigned char nMode)
{
	unsigned char i,j,k;
	//bound parameters to make sure bogus values don't send algorithm into la la land
	if (nG4>5) nG4 = 5;
	if (nG3>9) nG3 = 9;
	if (nG2>5) nG2 = 5;
	if (nG1>9) nG1 = 9;	
	if (nMode==DISP_LINEAR)
	{
		//Do red
		for (i = 0; i<6; i++)
		{
			if (i<nG4) nLeds[i] = LED_ON; else nLeds[i] = LED_OFF;
		}
		//Do Amber
		j = nG3 + 6;
		for(i =6; i<15; i++)
		{
			
			if (i<j) nLeds[i] = LED_ON; else nLeds[i] = LED_OFF;
		}
		//Do green
		j = nG2 + 15;
		for(i=15; i<21; i++)
		{
			
			if(i<j) nLeds[i] = LED_ON; else nLeds[i] = LED_OFF;
		}
		//Do Blue
		j = nG1 + 21;
		for(i=21;i<30;i++)
		{
			if(i<j) nLeds[i] = LED_ON; else nLeds[i] = LED_OFF;
		}
	}
	else
	{
		//Red
		for(i=0;i<6;i++) nLeds[i]=LED_OFF;
		j = nG4;
		for(i=0;i<j;i++)
		{
			k = rand() % 6;			//pick a random position
			if(nLeds[k]==LED_ON)	//already set
				i--;				//so try again till we've cleared j items
			else
				nLeds[k] = LED_ON;	//set this item
		}
		
		//amber
		for(i=6;i<15;i++) nLeds[i]=LED_OFF;
		j = nG3+6;
		for(i=6;i<j;i++)
		{
			k = rand() % 9;			//pick a random position
			k = k + 6;
			if(nLeds[k]==LED_ON)	//already set
				i--;				//so try again till we've set j items
			else
				nLeds[k] = LED_ON;	//set this item
		}
		
		//green
		for(i=15;i<21;i++) nLeds[i]=LED_OFF;
		j = nG2+15;
		for(i=15;i<j;i++)
		{
			k = rand() % 6;			//pick a random position
			k = k + 15;
			if(nLeds[k]==LED_ON)	//already set
				i--;				//so try again till we've set j items
			else
				nLeds[k] = LED_ON;	//clear this item
		}
		
		//mins
		for(i=21;i<30;i++) nLeds[i]=LED_OFF;
		j = nG1+21;
		for(i=21;i<j;i++)
		{
			k = rand() % 9;			//pick a random position
			k = k + 21;
			if(nLeds[k]==LED_ON)	//already set
				i--;				//so try again till we've set j items
			else
				nLeds[k] = LED_ON;	//set this item
		}	
				
	}
}

//FUNCTION to turn on all LEDs briefly
void flash_display(void)
{
	unsigned char i;
	for(i=0;i<30;i++) nLeds[i] = LED_ON;
	delay_ms(100);
	for(i=0;i<30;i++) nLeds[i] = LED_OFF;
}

//FUNCTION to flash all LEDs in a colour group briefly
//nGroup :- group to flash {0..3} 0=red, 1=yellow, 2=green, 3=blue
void flash_group(unsigned char nGroup)
{
	unsigned char i,j,k;
	switch(nGroup)
	{
		case 0:
			j = 0; k = 6;
			break;
		case 1:
			j = 6; k = 15;
			break;
		case 2:
			j = 15; k = 21;
			break;
		case 3:
			j = 21; k = 30;
			break;

	}
	for(i=j;i<k;i++) nLeds[i] = LED_ON;
	delay_ms(100);
	for(i=j;i<k;i++) nLeds[i] = LED_OFF;	
}