//*****************************************
// Microdot V1
// Copyright 2006 Philip Pulle
//                RGB Sunset Productions
// For private use only, not for commercial
// use. 
// ****************************************


#include <system.h>
#include "microdot.h"
#include "display.h"
#include "t1rtc.h"
#include "control.h"

// Set configuration word
#ifdef _PIC16
	#pragma DATA _CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_ON & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO 
	#pragma DATA _CONFIG2, _IESO_OFF & _FCMEN_OFF 
#endif //_PIC16

#pragma CLOCK_FREQ 8000000



void main()
{
	unsigned char i;
	unsigned short nDelay;
	
	t1con = 0;			//disable T1 external oscillator as quickly as possible for ICSP
	
	osccon = 0x76;
	while( (osccon & 0x04)==0);	//start 8MHz internal oscillator and wait for it to settle

	//setup peripherals
	porta = 0;
	portb = 0;
	cmcon = 0x07;		//disable comparator output, do now to set trisa properly
	trisa = 0xff;		//all input
	trisb = 0xff;		//all input
	ansel = 0x00;		//no analog inputs
	
	delay_s(2);			//allow programmer time to do stuff at startup
	
	led_test();			//cycle through all LEDs seperately
	
	init_display();		//startup the display interrupt
	init_rtc();			//startup the RTC
	RTCTick(0);			//reset the RTC timer variable
	
	//Main loop
	while(1)
	{
		scan_switches();	//always check the switches for input
		if (RTCTick(0))
		{
			//The RTC Tick variable has ticked down, so update the display to show the time
			calc_time();
			show_time(DISP_RANDOM);
			RTCTick(5);	//reset the tick to another 5 seconds
		}

	}
}
